import java.lang.*;

// 2D matrix table related variables
Table table;
int numRows, numCols;

// Storing the data in arrays to make the implementation easier 
int[] labelsCnt;
int[] checkOutCell; // map the log to a specific cell
int[] layer; // We have 10 layers each representing a layer of the deweyClass 0, 1, 2, 3, etc 

int minLabels = 0;
int maxLabels = 0; // will be calculated later

int minCheckOutRate = 0;
int maxCheckOutRate = 0; // will be calculate later
int maxCheckOutRatelog10 = 5;// will be calculated later

int minDewey = 0;
int maxDewey = 999;

// Drawing Parameters
float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;
float cellWidth, cellHeight;

int tableauHeight = 25;
int tableauWidth = 5;

// The tableuas
int[][] tableau0, tableau1, tableau2, tableau3, tableau4, tableau5, tableau6, tableau7, tableau8, tableau9;
// default is false
boolean tableau0B, tableau1B, tableau2B, tableau3B, tableau4B, tableau5B, tableau6B, tableau7B, tableau8B, tableau9B;


PFont helvFont;
// Legend Strings
String layer0 = "0- CS, info & generic";
String layer1 = "1- Phil & psyc";
String layer2 = "2- Religion";
String layer3 = "3- Social Sci";
String layer4 = "4- Language";
String layer5 = "5- Science";
String layer6 = "6- Tech";
String layer7 = "7- Arts & rec";
String layer8 = "8- Literature";
String layer9 = "9- History & geog";
   

int[] convert2Dto1D(int[][] twoDArray)
{
  int[] oneDArray = new int[tableauHeight*tableauWidth];
  int k = 0;
   for (int i = 0; i <tableauHeight; ++i)
  {
    for (int j = 0; j <tableauWidth; ++j)
    {
      oneDArray[k++] = twoDArray[i][j];
    }
  }
  
  
  return oneDArray;
}

void printTableau(int[][] tableau)
{
  
  for (int i = 0; i <tableauHeight; ++i)
  {
    StringBuilder line = new StringBuilder();
    for (int j = 0; j <tableauWidth; ++j)
    {
      line.append("  " + tableau[i][j]);
    }
    println(line.toString());
  }
}

int[][] constructLayerTableau(int layerNumber)
{
  int[][] layerTableau = new int[25][5];
  for (int i=0; i<numRows; i++)
  {
    if (layer[i] == layerNumber) //seperate only the layer that we are interested in
    {
      
     if (labelsCnt[i] <= 25)
        layerTableau[labelsCnt[i]-1][checkOutCell[i]] ++;
    }
  }
  
  return layerTableau;
}

void setup() {
  
  // Data Setting up
  size(1280, 720, P3D); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables.
   helvFont = createFont("Helvetica", 18); 

  // Make the screen resizable.                     
  surface.setResizable(true);

  //Put data into Processing table from csv
  table = loadTable("checkOutAndSubjectEntries.csv", "header");
  numRows = table.getRowCount(); 
  numCols = table.getColumnCount();
  println("Rows: " + numRows + " Columns: " + numCols);
  println("Trying out the ceil log function: " + ceil((float)Math.log10(11)));

  labelsCnt = new int[numRows];
 // checkOutRate = new int[numRows];

  //cellLogCheckOut = new int[numRows];
  
  // Calculate the corresponding cell for the checkOut and the layer for each deweyCLass 
  checkOutCell = new int[numRows];
  layer = new int[numRows];
  
  //Attributes are ordered as in Dewey, bibNumber, checkOutCount and SubjectEntriesCount
  // X-axis (Labels/Subject Entry Count): values at [][3]
  // Y-axis (Check Out Rate): values at [][2]
  // Colors in scatter plot reflect Dewey in [][0]
  
  for (int i=0; i<numRows; i++) {
     
     labelsCnt[i] = table.getInt(i, 3);
     layer[i] = floor(table.getInt(i, 0)/100);
     int temp = ceil((float)Math.log10(table.getInt(i, 2)));
     // Compute the corresponding cell for the check out rate 
     if(temp >= 0 && temp <= 1)
     {
       checkOutCell[i] = 0;
     }
     else
        checkOutCell[i] = temp -1;  
  }
     table.clearRows(); // save space
    
  cellWidth = (width - constantHorizontalMargin*2)/maxCheckOutRatelog10;
  cellHeight = (height - constantVerticalMargin*2)/tableauHeight;
  
  // Rendering Setup
  grid = new Cell[tableauHeight][maxCheckOutRatelog10];
  for (int i = 0; i < tableauHeight ; i++) {
    for (int j = 0; j < maxCheckOutRatelog10; j++) {
      // Initialize each object
      grid[i][j] = new Cell(constantHorizontalMargin + cellWidth*j,constantVerticalMargin 
        + cellHeight*i, cellWidth, cellHeight,i+j);
    }
  }



// Construct layer 0 tableau
tableau0 = constructLayerTableau(0);
println("Done with tableau 0" );
//printTableau(tableau0);


tableau1 = constructLayerTableau(1);
println("Done with tableau 1" );
//printTableau(tableau1);

tableau2 = constructLayerTableau(2);
println("Done with tableau 2" );
printTableau(tableau2);



tableau3 = constructLayerTableau(3);
println("Done with tableau 3" );
//printTableau(tableau3);



tableau4 = constructLayerTableau(4);
println("Done with tableau 4" );
//printTableau(tableau4);


tableau5 = constructLayerTableau(5);
println("Done with tableau 5" );
//printTableau(tableau5);

tableau6 = constructLayerTableau(6);
println("Done with tableau 6" );
//printTableau(tableau6);

tableau7 = constructLayerTableau(7);
println("Done with tableau 7" );
//printTableau(tableau7);

tableau8 = constructLayerTableau(8);
println("Done with tableau 8" );
//printTableau(tableau8);

tableau9 = constructLayerTableau(9);
println("Done with tableau 9" );
//printTableau(tableau9);



}


 


void draw()
{
  
  background(255);
  textFont(helvFont);
    
    if (tableau0B)
    {
    colorMode = 0;
    drawTableau(tableau0, 30, 20, 0);
    }
 
    if (tableau1B)
    {
    colorMode = 1;
    drawTableau(tableau1, 45, 20, 0);
    }

  if (tableau2B)
  {
    colorMode = 2;
    drawTableau(tableau2, 60, 20, 0);
  }

    if (tableau3B)
    {
    colorMode = 3;
    drawTableau(tableau3, 75, 20, 0);
    }
  
  if (tableau4B)
  {
    colorMode = 4;
    drawTableau(tableau4, 90, 20, 0);
  }
    if (tableau5B)
    {
    colorMode = 5;
    drawTableau(tableau5, 105, 20, 0);
    }
    
    if (tableau6B)
    {
    colorMode = 6;
    drawTableau(tableau6, 115, 20, 0);
    }
    
    if (tableau7B)
    {
    colorMode = 7;
    drawTableau(tableau7, 130, 20, 0);
    }
    if (tableau8B)
    {
    colorMode = 8;
    drawTableau(tableau8, 145, 20, 0);
    }
    if (tableau9B)
    {
    colorMode = 9;
    drawTableau(tableau9, 160, 20, 0);
    }
    
  
  
 textAlign(LEFT, CENTER);
  textSize(14);
  int y_start = 0;
  
  //Color = purple
  int rect_width = 20;
  int x_start = 980;
  
  //get All colors
  color[] purpleShades = getFiveShades(0);  color[] yellowShades = getFiveShades(5);
  color[] blueShades = getFiveShades(1);  color[] brownShades = getFiveShades(6);
  color[] pinkShades = getFiveShades(2);  color[] orangeShades = getFiveShades(7);
  color[] greenShades = getFiveShades(3);  color[] rainbowShades = getFiveShades(8);
  color[] redShades = getFiveShades(4);  color[] greyShades = getFiveShades(9);
  
  for (int i = 0; i<5; ++i)
  {
    fill(purpleShades[i]); rect( 1130 + (i*rect_width), 195, 20, 20);
  }
  text(layer0, x_start, y_start, 200, 400);
  
  //Color = blue
  for (int i = 0; i<5; ++i)
  {
    fill(blueShades[i]); rect( 1130 + (i*rect_width), 195 +20, 20, 20);
  }
  text(layer1, x_start, y_start + 20, 200, 400);
  
  //Color = pink
  for (int i = 0; i<5; ++i)
  {
    fill(pinkShades[i]); rect( 1130 + (i*rect_width), 195 +2*20, 20, 20);
  }
  text(layer2, x_start, y_start + 40, 200, 400);
  
  //Color = green
  for (int i = 0; i<5; ++i)
  {
    fill(greenShades[i]); rect( 1130 + (i*rect_width), 195 +3*20, 20, 20);
  }
  text(layer3, x_start, y_start + 60, 200, 400);
  //Color = red
  for (int i = 0; i<5; ++i)
  {
    fill(redShades[i]); rect( 1130 + (i*rect_width), 195 +4*20, 20, 20);
  }
  text(layer4, x_start, y_start + 80, 200, 400);
  //Color = yellow
  for (int i = 0; i<5; ++i)
  {
    fill(yellowShades[i]); rect( 1130 + (i*rect_width), 195 +5*20, 20, 20);
  }
  text(layer5, x_start, y_start + 100, 200, 400);
  //Color = brown
  for (int i = 0; i<5; ++i)
  {
    fill(brownShades[i]); rect( 1130 + (i*rect_width), 195 +6*20, 20, 20);
  }
  text(layer6, x_start, y_start + 120, 200, 400);
  //Color = orange
  for (int i = 0; i<5; ++i)
  {
    fill(orangeShades[i]); rect( 1130 + (i*rect_width), 195 +7*20, 20, 20);
  }
  text(layer7, x_start, y_start + 140, 200, 400);
  //Color = rainbow
  for (int i = 0; i<5; ++i)
  {
    fill(rainbowShades[i]); rect( 1130 + (i*rect_width), 195 +8*20, 20, 20);
  }
  text(layer8, x_start, y_start + 160, 200, 400);
  //Color = grey
  for (int i = 0; i<5; ++i)
  {
    fill(greyShades[i]); rect( 1130 + (i*rect_width), 195 +9*20, 20, 20);
  }
  text(layer9, x_start, y_start + 180, 200, 400);
  
  
  //Draw Title
  textAlign(CENTER, CENTER);
  fill(0);
  textSize(16);
  text("Number of labels [1, 25] vs log10(Checkout rate)", width/2, constantVerticalMargin/2);
  
  
   
     //Draw horizontal labels
  for (int i=1; i<tableauWidth; i++) {
    textAlign(TOP, CENTER);
    fill(0);
    textSize(16);
    text("10^" +i, constantHorizontalMargin + i*cellWidth -20 , constantVerticalMargin );
  }
   
   //Draw vertical labels
  for (int i=0; i<tableauHeight; i++) {
    textAlign(RIGHT, CENTER);
    fill(0);
    textSize(16);
    text(i+1, constantHorizontalMargin, constantVerticalMargin + cellHeight/2 + cellHeight*i);
  }
  
  // Draw robot
  //drawRobot();
  
  
  /*
  pushMatrix();
  translate(30, 20, -50);
  rect(0, 0, 55, 55);
  
  popMatrix();
  translate(30, 20, -50);
  rect(55, 0, 55, 55);
  */
  
  

  // The counter variables i and j are also the column and row numbers and 
  // are used as arguments to the constructor for each object in the grid.  
  
  
}